#include "output_writer.h"

using namespace std;

Output_writer::Output_writer() {
  os_statistics.open("output/statistics.out");
  
  for(int i = 0; i < 6; ++i) {
    global_statistics[i] = 0;
    local_statistics[i] = 0;
  }
}

Output_writer::~Output_writer() {
  write_statistics("TOTAL",global_statistics);
  os_statistics.close();
}

char const* Output_writer::get_ascii_string(char *const animal_string, 
                                            uint32_t const strength[USED_PIECES],
                                            int index, 
							                bool inversed) {
  animal_string[0] = '0';
  for(int i = 0; i < USED_PIECES; ++i) {
    int location = index % BOARD_SIZE;
	int actual_strength = strength[i];
	if(inversed) {
	  if(actual_strength % 2 == 0) {
		actual_strength+=1;
	  } else {
		actual_strength-=1;
	  }
	}
    index /= BOARD_SIZE;
    char ascii_loc[2] = {LOCATION_LABEL[location][0], LOCATION_LABEL[location][1]};
	
    animal_string[1+(2*actual_strength)]   = ascii_loc[0];
    animal_string[1+(2*actual_strength)+1] = ascii_loc[1];
	if(inversed) {
      animal_string[1+(2*actual_strength)+1] = inverseRank(ascii_loc[1]); 
	} 
  }
  return animal_string;
}

Move Output_writer::inverse(Move const &m){
  Move res = m;
  if(is_white(res.piece)) {
    res.piece++;
  } else {
	res.piece--;
  }
  res.from = get_location(LOCATION_LABEL[res.from][0],
						  inverseRank(LOCATION_LABEL[res.from][1]));
  res.to   = get_location(LOCATION_LABEL[res.to][0],
						  inverseRank(LOCATION_LABEL[res.to][1]));
  
  return res;
}

char Output_writer::inverseRank(char rank) {
  return (BOARD_HEIGHT - (rank - '0') + 1) + '0';
}

void Output_writer::write_line(History const & history,
							   uint32_t const strength[USED_PIECES],
                               int index, 
							   bool inversed) {
  char empty_string[] = "0xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"; 
  char const* ascii_string = get_ascii_string(empty_string,strength,index,inversed);
  Position board;
  
  // additional check for stalemates
  set_configuration(board,ascii_string);
  if(is_stalemate(board)) {
    return;
  }
  if(board.is_won()){
	cerr << "Trying to add a winning board (" << ascii_string << ") to database. " << endl;
    return;
  }
  
  if(inversed) {
    cout << (history.calculated ? (history.winner ? "w" : "b") : "d") << " ";
    if(history.calculated) {
      write_move(inverse(history.best_move), stdout);
	} else {
	  cout << "?";
	}
  } else {
    cout << (history.calculated ? (history.winner ? "b" : "w") : "d") << " ";
    if(history.calculated) {
      write_move(history.best_move, stdout);
	} else {
	  cout << "?";
	}
  }
  
  cout << " " << history.nr_of_moves << " " << ascii_string << "\n";
}

void Output_writer::write_statistics(string const headline,
                                     uint32_t const * const statistics) {
  os_statistics << headline << endl
    << "-----------------------" << endl 
    << "total     : " << statistics[0] << endl
    << "white wins: " << statistics[2] << " (" << statistics[3] << ")" << endl
    << "black wins: " << statistics[4] << " (" << statistics[5] << ")" << endl
    << "draws     : " << statistics[1] << endl
    << "=======================" << "\n";
}

string Output_writer::strength_to_animalname(uint32_t strength) {
  if(strength < 2)
    return "rat";
  else if(strength < 4)
    return "cat";
  else if(strength < 6)
    return "wolf";
  else if(strength < 8)
    return "dog";
  else if(strength < 10)
    return "panther";
  else if(strength < 12)
    return "tiger";
  else if(strength < 14)
    return "lion";
  else
    return "elephant";
}

string Output_writer::strength_to_string(uint32_t const strength[],int const used_pieces) {
  string white = "white";
  string black = "black";
  for(int i = 0; i < used_pieces; ++i) {
    if(strength[i]%2 == 0) {
      white += " " + strength_to_animalname(strength[i]);
    } else {
      black += " " + strength_to_animalname(strength[i]);
    }
  }
  return white + " VS. " + black;
}

void Output_writer::flush_statistics(uint32_t const strength[], 
                                     int const used_pieces) {
  write_statistics(strength_to_string(strength,used_pieces),local_statistics);
  for(int i = 0; i < 6; ++i) {
    local_statistics[i] = 0;
  }
}

void Output_writer::increment_statistic(int index) {
  local_statistics[index] += 1;
  global_statistics[index] += 1;
}
