// *******************************************************************
//   (C) Copyright 2013 Leiden Institute of Advanced Computer Science
//   Universiteit Leiden
//   All Rights Reserved
// *******************************************************************
// Output writer
// *******************************************************************
// FILE INFORMATION:
//   File:     output_writer.h
//   Author:   Jan N. van Rijn
//   Revision: 0.01a
//   Date:     2013/03/13
// *******************************************************************
// DESCRIPTION: Output writer. Generates output that might be
// meaningfull for analyses. 
// *******************************************************************

#if !defined(__output_writer_h__)
#define __output_writer_h__

#include "../core/move.h"
#include "../core/position.h"
#include "../interface/cstdio_interface.h"
#include "../interface/core_extensions.h"

#include "constants.h"
#include "history.h"

#include <cstdlib>
#include <iostream>
#include <string>
#include <fstream>

class Output_writer {
  public:
    Output_writer();
    ~Output_writer();
    void write_line(History const & history,uint32_t const strength[],int index,bool inversed);
    void flush_statistics(uint32_t const strength[], int const);
	std::string strength_to_string(uint32_t const strength[], int const);
  private:
    void write_statistics(std::string const,uint32_t const * const);
    void increment_statistic(int);
	Move inverse(Move const &);
    
    std::string strength_to_animalname(uint32_t);
	char const* get_ascii_string(char *const string, 
                                 uint32_t const strength[],
                                 int index, 
							     bool inversed);
	char inverseRank(char rank);
    std::ofstream os_statistics;
    // {games,draws,white_wins,white_to_play_wins,black_wins,black_to_play_wins} 
    uint32_t global_statistics[6]; 
    uint32_t  local_statistics[6];
};

#endif
